@@ -26,6 +26,7 @@ from sqlalchemy import types
 from sqlalchemy import Unicode
 from sqlalchemy import UniqueConstraint
 from sqlalchemy import VARCHAR
+from sqlalchemy.dialects.mysql import LONGTEXT
 from sqlalchemy.engine.default import DefaultDialect
 from sqlalchemy.sql import and_
 from sqlalchemy.sql import column
@@ -53,7 +54,6 @@ from alembic.util import sqla_compat
 
 
 class AutogenRenderTest(TestBase):
-
     """test individual directives"""
 
     def setUp(self):
@@ -94,6 +94,46 @@ class AutogenRenderTest(TestBase):
             "['active', 'code'], unique=False)",
         )
 
+    def test_render_add_index_fn(self):
+        t = self.table(Column("other", String(100)))
+        idx = Index("test_fn_idx", t.c.code + t.c.other)
+        op_obj = ops.CreateIndexOp.from_index(idx)
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.create_index('test_fn_idx', 'test', "
+            "[sa.literal_column('code || other')], unique=False)",
+        )
+
+    def test_render_add_index_label(self):
+        t = self.table(Column("other", String(100)))
+        idx = Index(
+            "test_fn_idx",
+            (t.c.code + t.c.other).label("foo"),
+            t.c.id.label("bar"),
+        )
+        op_obj = ops.CreateIndexOp.from_index(idx)
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.create_index('test_fn_idx', 'test', ["
+            "sa.literal_column('code || other').label('foo'), "
+            "sa.literal_column('id').label('bar')"
+            "], unique=False)",
+        )
+
+    def test_render_add_index_if_not_exists(self):
+        """
+        autogenerate.render._add_index
+        """
+        t = self.table()
+        idx = Index("test_active_code_idx", t.c.active, t.c.code)
+        op_obj = ops.CreateIndexOp.from_index(idx)
+        op_obj.if_not_exists = True
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.create_index('test_active_code_idx', 'test', "
+            "['active', 'code'], unique=False, if_not_exists=True)",
+        )
+
     @testing.emits_warning("Can't validate argument ")
     def test_render_add_index_custom_kwarg(self):
         t = self.table()
@@ -156,7 +196,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_index('test_active_code_idx', 'test', "
-            "['active', sa.text('lower(code)')], unique=False)",
+            "['active', sa.literal_column('lower(code)')], unique=False)",
         )
         op_obj_rev = op_obj.reverse()
         eq_ignore_whitespace(
@@ -172,7 +212,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_index('test_lower_code_idx', 'test', "
-            "[sa.text('lower(code)')], unique=False)",
+            "[sa.literal_column('lower(code)')], unique=False)",
         )
         op_obj_rev = op_obj.reverse()
         eq_ignore_whitespace(
@@ -188,7 +228,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_index('test_lower_code_idx', 'test', "
-            "[sa.text('CAST(code AS VARCHAR)')], unique=False)",
+            "[sa.literal_column('CAST(code AS VARCHAR)')], unique=False)",
         )
 
     def test_render_add_index_desc(self):
@@ -198,7 +238,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_index('test_desc_code_idx', 'test', "
-            "[sa.text('code DESC')], unique=False)",
+            "[sa.literal_column('code DESC')], unique=False)",
         )
 
     def test_drop_index(self):
@@ -213,6 +253,20 @@ class AutogenRenderTest(TestBase):
             "op.drop_index('test_active_code_idx', table_name='test')",
         )
 
+    def test_drop_index_if_exists(self):
+        """
+        autogenerate.render._drop_index
+        """
+        t = self.table()
+        idx = Index("test_active_code_idx", t.c.active, t.c.code)
+        op_obj = ops.DropIndexOp.from_index(idx)
+        op_obj.if_exists = True
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.drop_index('test_active_code_idx', table_name='test', "
+            "if_exists=True)",
+        )
+
     def test_drop_index_text(self):
         """
         autogenerate.render._drop_index
@@ -228,7 +282,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj_rev),
             "op.create_index('test_active_code_idx', 'test', "
-            "['active', sa.text('lower(code)')], unique=False)",
+            "['active', sa.literal_column('lower(code)')], unique=False)",
         )
 
     def test_drop_index_func(self):
@@ -246,7 +300,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj_rev),
             "op.create_index('test_lower_code_idx', 'test', "
-            "[sa.text('lower(code)')], unique=False)",
+            "[sa.literal_column('lower(code)')], unique=False)",
         )
 
     @testing.emits_warning("Can't validate argument ")
@@ -990,6 +1044,19 @@ class AutogenRenderTest(TestBase):
             "mysql_engine='InnoDB',sqlite_autoincrement=True)",
         )
 
+    def test_render_if_not_exists(self):
+        t = self.table()
+        op_obj = ops.CreateTableOp.from_table(t)
+        op_obj.if_not_exists = True
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.create_table('test',"
+            "sa.Column('id', sa.Integer(), nullable=False),"
+            "sa.Column('active', sa.Boolean(), nullable=True),"
+            "sa.Column('code', sa.String(length=255), nullable=True),"
+            "sa.PrimaryKeyConstraint('id'),if_not_exists=True)",
+        )
+
     def test_render_drop_table(self):
         op_obj = ops.DropTableOp.from_table(Table("sometable", MetaData()))
         eq_ignore_whitespace(
@@ -1006,6 +1073,15 @@ class AutogenRenderTest(TestBase):
             "op.drop_table('sometable', schema='foo')",
         )
 
+    def test_render_drop_table_if_exists(self):
+        t = self.table()
+        op_obj = ops.DropTableOp.from_table(t)
+        op_obj.if_exists = True
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.drop_table('test', if_exists=True)",
+        )
+
     def test_render_table_no_implicit_check(self):
         m = MetaData()
         t = Table("test", m, Column("x", Boolean()))
@@ -1251,6 +1327,18 @@ class AutogenRenderTest(TestBase):
             {"from mypackage import MySpecialType"},
         )
 
+    def test_render_modify_name(self):
+        op_obj = ops.AlterColumnOp(
+            "sometable",
+            "somecolumn",
+            modify_name="newcolumnname",
+        )
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.alter_column('sometable', 'somecolumn', "
+            "new_column_name='newcolumnname')",
+        )
+
     def test_render_modify_type(self):
         op_obj = ops.AlterColumnOp(
             "sometable",
@@ -1618,28 +1706,36 @@ class AutogenRenderTest(TestBase):
             "existing_server_default='5')",
         )
 
+    def _check_enum_inherit_schema(self, enum):
+        if enum.inherit_schema:
+            return enum, ", inherit_schema=True"
+        else:
+            return enum, ""
+
     def test_render_enum(self):
-        eq_ignore_whitespace(
-            autogenerate.render._repr_type(
-                Enum("one", "two", "three", name="myenum"),
-                self.autogen_context,
-            ),
-            "sa.Enum('one', 'two', 'three', name='myenum')",
+        enum, extra = self._check_enum_inherit_schema(
+            Enum("one", "two", "three", name="myenum")
         )
         eq_ignore_whitespace(
-            autogenerate.render._repr_type(
-                Enum("one", "two", "three"), self.autogen_context
-            ),
-            "sa.Enum('one', 'two', 'three')",
+            autogenerate.render._repr_type(enum, self.autogen_context),
+            f"sa.Enum('one', 'two', 'three', name='myenum'{extra})",
+        )
+
+        enum, extra = self._check_enum_inherit_schema(
+            Enum("one", "two", "three")
+        )
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(enum, self.autogen_context),
+            f"sa.Enum('one', 'two', 'three'{extra})",
         )
 
     def test_render_non_native_enum(self):
+        enum, extra = self._check_enum_inherit_schema(
+            Enum("one", "two", "three", native_enum=False)
+        )
         eq_ignore_whitespace(
-            autogenerate.render._repr_type(
-                Enum("one", "two", "three", native_enum=False),
-                self.autogen_context,
-            ),
-            "sa.Enum('one', 'two', 'three', native_enum=False)",
+            autogenerate.render._repr_type(enum, self.autogen_context),
+            f"sa.Enum('one', 'two', 'three'{extra}, native_enum=False)",
         )
 
     def test_repr_plain_sqla_type(self):
@@ -1793,7 +1889,6 @@ class AutogenRenderTest(TestBase):
             .with_variant(CHAR(15), "oracle")
         )
 
-        # the new Black formatting will help a lot with this
         eq_ignore_whitespace(
             autogenerate.render._repr_type(type_, self.autogen_context),
             "sa.String(length=5)."
@@ -1801,6 +1896,18 @@ class AutogenRenderTest(TestBase):
             "with_variant(sa.CHAR(length=15), 'oracle')",
         )
 
+    def test_render_reverse_variant(self):
+        """test #1585"""
+
+        self.autogen_context.opts["user_module_prefix"] = None
+
+        type_ = LONGTEXT().with_variant(String(10), "oracle")
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(type_, self.autogen_context),
+            "mysql.LONGTEXT()." "with_variant(sa.String(length=10), 'oracle')",
+        )
+
     def test_repr_user_type_user_prefix_None(self):
         class MyType(UserDefinedType):
             def get_col_spec(self):
@@ -2070,6 +2177,38 @@ class AutogenRenderTest(TestBase):
             ")",
         )
 
+    @testing.combinations((True,), (False,), argnames="use_schema")
+    def test_render_create_table_comment_op_batch(self, use_schema):
+        """test #1361"""
+        uo = ops.UpgradeOps(
+            ops=[
+                ops.ModifyTableOps(
+                    "table_name",
+                    schema="SomeSchema" if use_schema else None,
+                    ops=[
+                        ops.CreateTableCommentOp(
+                            "table_name",
+                            "comment",
+                            schema="SomeSchema" if use_schema else None,
+                        )
+                    ],
+                )
+            ]
+        )
+
+        eq_(
+            autogenerate.render_python_code(uo, render_as_batch=True),
+            "# ### commands auto generated by Alembic - please adjust! ###\n"
+            "    with op.batch_alter_table('table_name', "
+            f"schema={repr('SomeSchema' if use_schema else None)}) "
+            "as batch_op:\n"
+            "        batch_op.create_table_comment(\n"
+            "        'comment',\n"
+            "        existing_comment=None\n"
+            "    )\n\n"
+            "    # ### end Alembic commands ###",
+        )
+
     def test_render_drop_table_comment_op(self):
         op_obj = ops.DropTableCommentOp("table_name")
         eq_ignore_whitespace(
@@ -2081,6 +2220,36 @@ class AutogenRenderTest(TestBase):
             ")",
         )
 
+    @testing.combinations((True,), (False,), argnames="use_schema")
+    def test_render_drop_table_comment_op_batch(self, use_schema):
+        """test #1361"""
+        uo = ops.UpgradeOps(
+            ops=[
+                ops.ModifyTableOps(
+                    "table_name",
+                    schema="SomeSchema" if use_schema else None,
+                    ops=[
+                        ops.DropTableCommentOp(
+                            "table_name",
+                            schema="SomeSchema" if use_schema else None,
+                        )
+                    ],
+                )
+            ]
+        )
+
+        eq_(
+            autogenerate.render_python_code(uo, render_as_batch=True),
+            "# ### commands auto generated by Alembic - please adjust! ###\n"
+            "    with op.batch_alter_table('table_name', "
+            f"schema={repr('SomeSchema' if use_schema else None)}) "
+            "as batch_op:\n"
+            "        batch_op.drop_table_comment(\n"
+            "        existing_comment=None\n"
+            "    )\n\n"
+            "    # ### end Alembic commands ###",
+        )
+
     def test_render_drop_table_comment_op_existing_with_quote(self):
         op_obj = ops.DropTableCommentOp(
             "table_name", existing_comment="This was john's comment"
@@ -2094,7 +2263,6 @@ class AutogenRenderTest(TestBase):
             ")",
         )
 
-    @config.requirements.computed_columns_api
     def test_render_add_column_computed(self):
         c = sa.Computed("5")
         op_obj = ops.AddColumnOp("foo", Column("x", Integer, c))
@@ -2104,7 +2272,6 @@ class AutogenRenderTest(TestBase):
             "sa.Computed('5', ), nullable=True))",
         )
 
-    @config.requirements.computed_columns_api
     @testing.combinations((True,), (False,))
     def test_render_add_column_computed_persisted(self, persisted):
         op_obj = ops.AddColumnOp(
@@ -2116,7 +2283,6 @@ class AutogenRenderTest(TestBase):
             "sa.Computed('5', persisted=%s), nullable=True))" % persisted,
         )
 
-    @config.requirements.computed_columns_api
     def test_render_alter_column_computed_modify_default(self):
         op_obj = ops.AlterColumnOp(
             "sometable", "somecolumn", modify_server_default=sa.Computed("7")
@@ -2127,7 +2293,6 @@ class AutogenRenderTest(TestBase):
             "server_default=sa.Computed('7', ))",
         )
 
-    @config.requirements.computed_columns_api
     def test_render_alter_column_computed_existing_default(self):
         op_obj = ops.AlterColumnOp(
             "sometable",
@@ -2140,7 +2305,6 @@ class AutogenRenderTest(TestBase):
             "existing_server_default=sa.Computed('42', ))",
         )
 
-    @config.requirements.computed_columns_api
     @testing.combinations((True,), (False,))
     def test_render_alter_column_computed_modify_default_persisted(
         self, persisted
@@ -2156,7 +2320,6 @@ class AutogenRenderTest(TestBase):
             "=sa.Computed('7', persisted=%s))" % persisted,
         )
 
-    @config.requirements.computed_columns_api
     @testing.combinations((True,), (False,))
     def test_render_alter_column_computed_existing_default_persisted(
         self, persisted
@@ -2209,7 +2372,6 @@ class AutogenRenderTest(TestBase):
         ),
     )
 
-    @config.requirements.identity_columns_api
     @identity_comb
     def test_render_add_column_identity(self, kw, text):
         col = Column("x", Integer, sa.Identity(**kw))
@@ -2220,7 +2382,6 @@ class AutogenRenderTest(TestBase):
             "%s, nullable=%r))" % (text, col.nullable),
         )
 
-    @config.requirements.identity_columns_api
     @identity_comb
     def test_render_alter_column_add_identity(self, kw, text):
         op_obj = ops.AlterColumnOp(
@@ -2236,7 +2397,6 @@ class AutogenRenderTest(TestBase):
             "server_default=%s)" % text,
         )
 
-    @config.requirements.identity_columns_api
     def test_render_alter_column_drop_identity(self):
         op_obj = ops.AlterColumnOp(
             "foo",
@@ -2289,11 +2449,11 @@ class RenderNamingConventionTest(TestBase):
         t = Table("t", self.metadata, Column("c", Integer))
         eq_ignore_whitespace(
             autogenerate.render._render_unique_constraint(
-                UniqueConstraint(t.c.c, deferrable="XYZ"),
+                UniqueConstraint(t.c.c, deferrable=True),
                 self.autogen_context,
                 None,
             ),
-            "sa.UniqueConstraint('c', deferrable='XYZ', "
+            "sa.UniqueConstraint('c', deferrable=True, "
             "name=op.f('uq_ct_t_c'))",
         )
 
@@ -2387,6 +2547,18 @@ class RenderNamingConventionTest(TestBase):
             "name=op.f('fk_ct_t_c_q')))",
         )
 
+    def test_added_fk(self):
+        t = Table("t", self.metadata, Column("c", Integer, ForeignKey("q.id")))
+
+        fk = list(t.foreign_key_constraints)[0]
+        eq_ignore_whitespace(
+            autogenerate.render._render_foreign_key(
+                fk, self.autogen_context, self.metadata
+            ),
+            "sa.ForeignKeyConstraint(['c'], ['q.id'], "
+            "name=op.f('fk_ct_t_c_q'))",
+        )
+
     def test_render_check_constraint_renamed(self):
         """test that constraints from autogenerate render with
         the naming convention name explicitly.  These names should
