@@ -26,6 +26,7 @@ from alembic.testing import is_false
 from alembic.testing import is_true
 from alembic.testing import mock
 from alembic.testing.env import _get_staging_directory
+from alembic.testing.env import _multidb_testing_config
 from alembic.testing.env import _no_sql_testing_config
 from alembic.testing.env import _sqlite_file_db
 from alembic.testing.env import _sqlite_testing_config
@@ -63,7 +64,7 @@ class HistoryTest(_BufMixin, TestBase):
     def teardown_class(cls):
         clear_staging_env()
 
-    def teardown(self):
+    def tearDown(self):
         self.cfg.set_main_option("revision_environment", "false")
 
     @classmethod
@@ -205,13 +206,12 @@ finally:
 
 
 class RevisionEnvironmentTest(_BufMixin, TestBase):
-    @classmethod
-    def setup(cls):
-        cls.env = staging_env()
-        cls.cfg = _sqlite_testing_config()
-        cls._setup_env_file()
+    def setUp(self):
+        self.env = staging_env()
+        self.cfg = _sqlite_testing_config()
+        self._setup_env_file()
 
-    def teardown(self):
+    def tearDown(self):
         self.cfg.set_main_option("revision_environment", "false")
         clear_staging_env()
 
@@ -322,12 +322,6 @@ class CurrentTest(_BufMixin, TestBase):
         with self._assert_lines(["a3"]):
             command.current(self.cfg)
 
-    def test_current_obfuscate_password(self):
-        eq_(
-            util.obfuscate_url_pw("postgresql://scott:tiger@localhost/test"),
-            "postgresql://scott:***@localhost/test",
-        )
-
     def test_two_heads(self):
         command.stamp(self.cfg, ())
         command.stamp(self.cfg, (self.a1.revision, self.b1.revision))
@@ -618,20 +612,22 @@ class CheckTest(TestBase):
 
     def _env_fixture(self, version_table_pk=True):
         env_file_fixture(
-            """
+            f"""
 
 from sqlalchemy import MetaData, engine_from_config
 target_metadata = MetaData()
 
 engine = engine_from_config(
     config.get_section(config.config_ini_section),
-    prefix='sqlalchemy.')
+    prefix='sqlalchemy.'
+)
 
 connection = engine.connect()
 
 context.configure(
-    connection=connection, target_metadata=target_metadata,
-    version_table_pk=%r
+    connection=connection,
+    target_metadata=target_metadata,
+    version_table_pk={version_table_pk}
 )
 
 try:
@@ -642,7 +638,6 @@ finally:
     engine.dispose()
 
 """
-            % (version_table_pk,)
         )
 
     def test_check_no_changes(self):
@@ -665,6 +660,53 @@ finally:
             )
 
 
+class CheckTestMultiDB(CheckTest):
+    def setUp(self):
+        self.engine1 = _sqlite_file_db(tempname="eng1.db")
+        self.engine2 = _sqlite_file_db(tempname="eng2.db")
+        self.engine3 = _sqlite_file_db(tempname="eng3.db")
+
+        self.env = staging_env(template="multidb")
+        self.cfg = _multidb_testing_config(
+            {
+                "engine1": self.engine1,
+                "engine2": self.engine2,
+                "engine3": self.engine3,
+            }
+        )
+
+    def _env_fixture(self):
+        env_file_fixture(
+            """
+
+import re
+from sqlalchemy import MetaData, engine_from_config
+
+db_names = config.get_main_option("databases", "")
+for db_name in re.split(r",\\s*", db_names):
+    engine = engine_from_config(
+        config.get_section(db_name),
+        prefix="sqlalchemy.",
+    )
+    connection = engine.connect()
+    metadata = MetaData()
+    context.configure(
+        connection=connection,
+        target_metadata=metadata,
+    )
+
+    try:
+        with context.begin_transaction():
+            context.run_migrations()
+    finally:
+        connection.close()
+        engine.dispose()
+
+
+"""
+        )
+
+
 class _StampTest:
     def _assert_sql(self, emitted_sql, origin, destinations):
         ins_expr = (
@@ -1102,7 +1144,7 @@ class CommandLineTest(TestBase):
         cls.cfg = _sqlite_testing_config()
         cls.a, cls.b, cls.c = three_rev_fixture(cls.cfg)
 
-    def teardown(self):
+    def tearDown(self):
         os.environ.pop("ALEMBIC_CONFIG", None)
 
     @classmethod
@@ -1185,9 +1227,12 @@ class CommandLineTest(TestBase):
         assert not commands, "Commands without help text: %s" % commands
 
     def test_init_file_exists_and_is_not_empty(self):
-        with mock.patch(
-            "alembic.command.os.listdir", return_value=["file1", "file2"]
-        ), mock.patch("alembic.command.os.access", return_value=True):
+        with (
+            mock.patch(
+                "alembic.command.os.listdir", return_value=["file1", "file2"]
+            ),
+            mock.patch("alembic.command.os.access", return_value=True),
+        ):
             directory = "alembic"
             assert_raises_message(
                 util.CommandError,
@@ -1244,12 +1289,11 @@ class CommandLineTest(TestBase):
             else:
                 return ["file1", "file2", "alembic.ini.mako"]
 
-        with mock.patch(
-            "alembic.command.os.access", side_effect=access_
-        ), mock.patch("alembic.command.os.makedirs") as makedirs, mock.patch(
-            "alembic.command.os.listdir", side_effect=listdir_
-        ), mock.patch(
-            "alembic.command.ScriptDirectory"
+        with (
+            mock.patch("alembic.command.os.access", side_effect=access_),
+            mock.patch("alembic.command.os.makedirs") as makedirs,
+            mock.patch("alembic.command.os.listdir", side_effect=listdir_),
+            mock.patch("alembic.command.ScriptDirectory"),
         ):
             command.init(self.cfg, directory="foobar")
             eq_(
@@ -1264,10 +1308,10 @@ class CommandLineTest(TestBase):
             else:
                 return False
 
-        with mock.patch(
-            "alembic.command.os.access", side_effect=access_
-        ), mock.patch("alembic.command.os.makedirs") as makedirs, mock.patch(
-            "alembic.command.ScriptDirectory"
+        with (
+            mock.patch("alembic.command.os.access", side_effect=access_),
+            mock.patch("alembic.command.os.makedirs") as makedirs,
+            mock.patch("alembic.command.ScriptDirectory"),
         ):
             command.init(self.cfg, directory="foobar")
             eq_(
