@@ -102,7 +102,7 @@ class ConfigTest(TestBase):
         cfg.attributes["connection"] = m1
         eq_(cfg.attributes["connection"], m1)
 
-    def test_attributes_construtor(self):
+    def test_attributes_constructor(self):
         m1 = mock.Mock()
         m2 = mock.Mock()
         cfg = config.Config(attributes={"m1": m1})
@@ -134,6 +134,12 @@ class ConfigTest(TestBase):
             "/foo /bar",
             ["/foo", "/bar"],
         ),
+        (
+            "multiline string 1",
+            "newline",
+            " /foo  \n/bar  ",
+            ["/foo", "/bar"],
+        ),
         (
             "Linux pathsep 1",
             ":",
@@ -171,7 +177,7 @@ class ConfigTest(TestBase):
             "/foo|/bar",
             ValueError(
                 "'|' is not a valid value for version_path_separator; "
-                "expected 'space', 'os', ':', ';'"
+                "expected 'space', 'newline', 'os', ':', ';'"
             ),
         ),
         id_="iaaa",
@@ -188,7 +194,8 @@ class ConfigTest(TestBase):
         cfg.set_main_option("version_locations", string_value)
 
         if isinstance(expected_result, ValueError):
-            with expect_raises_message(ValueError, expected_result.args[0]):
+            message = str(expected_result)
+            with expect_raises_message(ValueError, message, text_exact=True):
                 ScriptDirectory.from_config(cfg)
         else:
             s = ScriptDirectory.from_config(cfg)
