@@ -1,7 +1,11 @@
 from sqlalchemy import Boolean
 from sqlalchemy import Column
+from sqlalchemy import Computed
 from sqlalchemy import DATETIME
+from sqlalchemy import exc
+from sqlalchemy import Float
 from sqlalchemy import func
+from sqlalchemy import Identity
 from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
@@ -11,19 +15,21 @@ from sqlalchemy import TIMESTAMP
 
 from alembic import op
 from alembic import util
+from alembic.autogenerate import api
+from alembic.autogenerate import compare
 from alembic.migration import MigrationContext
+from alembic.operations import ops
 from alembic.testing import assert_raises_message
+from alembic.testing import combinations
 from alembic.testing import config
 from alembic.testing.env import clear_staging_env
 from alembic.testing.env import staging_env
 from alembic.testing.fixtures import AlterColRoundTripFixture
 from alembic.testing.fixtures import op_fixture
 from alembic.testing.fixtures import TestBase
-from alembic.util import sqla_compat
 
 
 class MySQLOpTest(TestBase):
-    @config.requirements.comments_api
     def test_create_table_with_comment(self):
         context = op_fixture("mysql")
         op.create_table(
@@ -33,7 +39,6 @@ class MySQLOpTest(TestBase):
         )
         context.assert_contains("COMMENT='This is a table comment'")
 
-    @config.requirements.comments_api
     def test_create_table_with_column_comments(self):
         context = op_fixture("mysql")
         op.create_table(
@@ -52,7 +57,6 @@ class MySQLOpTest(TestBase):
             "COMMENT='This is a table comment'"
         )
 
-    @config.requirements.comments_api
     def test_add_column_with_comment(self):
         context = op_fixture("mysql")
         op.add_column("t", Column("q", Integer, comment="This is a comment"))
@@ -197,6 +201,35 @@ class MySQLOpTest(TestBase):
             "ALTER TABLE t CHANGE c c DATETIME NULL DEFAULT CURRENT_TIMESTAMP"
         )
 
+    def test_alter_column_modify_programmatic_default(self):
+        # test issue #736
+        # when autogenerate.compare creates the operation object
+        # programmatically, the server_default of the op has the full
+        # DefaultClause present.   make sure the usual renderer works.
+        context = op_fixture("mysql")
+
+        m1 = MetaData()
+
+        autogen_context = api.AutogenContext(context, m1)
+
+        operation = ops.AlterColumnOp("t", "c")
+        for fn in (
+            compare._compare_nullable,
+            compare._compare_type,
+            compare._compare_server_default,
+        ):
+            fn(
+                autogen_context,
+                operation,
+                None,
+                "t",
+                "c",
+                Column("c", Float(), nullable=False, server_default=text("0")),
+                Column("c", Float(), nullable=True, default=0),
+            )
+        op.invoke(operation)
+        context.assert_("ALTER TABLE t MODIFY c FLOAT NULL DEFAULT 0")
+
     def test_col_not_nullable(self):
         context = op_fixture("mysql")
         op.alter_column("t1", "c1", nullable=False, existing_type=Integer)
@@ -248,7 +281,6 @@ class MySQLOpTest(TestBase):
             server_default="q",
         )
 
-    @config.requirements.comments_api
     def test_alter_column_add_comment(self):
         context = op_fixture("mysql")
         op.alter_column(
@@ -264,7 +296,6 @@ class MySQLOpTest(TestBase):
             "COMMENT 'This is a column comment'"
         )
 
-    @config.requirements.comments_api
     def test_alter_column_add_comment_quoting(self):
         context = op_fixture("mysql")
         op.alter_column(
@@ -280,7 +311,6 @@ class MySQLOpTest(TestBase):
             "COMMENT 'This is a ''column'' comment'"
         )
 
-    @config.requirements.comments_api
     def test_alter_column_drop_comment(self):
         context = op_fixture("mysql")
         op.alter_column(
@@ -294,7 +324,6 @@ class MySQLOpTest(TestBase):
 
         context.assert_("ALTER TABLE foo.t MODIFY c BOOL NULL")
 
-    @config.requirements.comments_api
     def test_alter_column_existing_comment(self):
         context = op_fixture("mysql")
         op.alter_column(
@@ -310,7 +339,6 @@ class MySQLOpTest(TestBase):
             "COMMENT 'existing column comment'"
         )
 
-    @config.requirements.comments_api
     def test_rename_column_existing_comment(self):
         context = op_fixture("mysql")
         op.alter_column(
@@ -327,7 +355,6 @@ class MySQLOpTest(TestBase):
             "COMMENT 'existing column comment'"
         )
 
-    @config.requirements.comments_api
     def test_alter_column_new_comment_replaces_existing(self):
         context = op_fixture("mysql")
         op.alter_column(
@@ -344,27 +371,23 @@ class MySQLOpTest(TestBase):
             "COMMENT 'This is a column comment'"
         )
 
-    @config.requirements.comments_api
     def test_create_table_comment(self):
         # this is handled by SQLAlchemy's compilers
         context = op_fixture("mysql")
         op.create_table_comment("t2", comment="t2 table", schema="foo")
         context.assert_("ALTER TABLE foo.t2 COMMENT 't2 table'")
 
-    @config.requirements.comments_api
-    @config.requirements.sqlalchemy_issue_4436
     def test_drop_table_comment(self):
         # this is handled by SQLAlchemy's compilers
         context = op_fixture("mysql")
         op.drop_table_comment("t2", existing_comment="t2 table", schema="foo")
         context.assert_("ALTER TABLE foo.t2 COMMENT ''")
 
-    @config.requirements.computed_columns_api
     def test_add_column_computed(self):
         context = op_fixture("mysql")
         op.add_column(
             "t1",
-            Column("some_column", Integer, sqla_compat.Computed("foo * 5")),
+            Column("some_column", Integer, Computed("foo * 5")),
         )
         context.assert_(
             "ALTER TABLE t1 ADD COLUMN some_column "
@@ -372,13 +395,19 @@ class MySQLOpTest(TestBase):
         )
 
     def test_drop_fk(self):
+        context = op_fixture("mysql")
+        op.drop_constraint("f1", "t1", type_="foreignkey")
+        context.assert_("ALTER TABLE t1 DROP FOREIGN KEY f1")
+
+    def test_drop_fk_legacy(self):
+        """#1245"""
         context = op_fixture("mysql")
         op.drop_constraint("f1", "t1", "foreignkey")
         context.assert_("ALTER TABLE t1 DROP FOREIGN KEY f1")
 
     def test_drop_fk_quoted(self):
         context = op_fixture("mysql")
-        op.drop_constraint("MyFk", "MyTable", "foreignkey")
+        op.drop_constraint("MyFk", "MyTable", type_="foreignkey")
         context.assert_("ALTER TABLE `MyTable` DROP FOREIGN KEY `MyFk`")
 
     def test_drop_constraint_primary(self):
@@ -388,32 +417,32 @@ class MySQLOpTest(TestBase):
 
     def test_drop_unique(self):
         context = op_fixture("mysql")
-        op.drop_constraint("f1", "t1", "unique")
+        op.drop_constraint("f1", "t1", type_="unique")
         context.assert_("ALTER TABLE t1 DROP INDEX f1")
 
     def test_drop_unique_quoted(self):
         context = op_fixture("mysql")
-        op.drop_constraint("MyUnique", "MyTable", "unique")
+        op.drop_constraint("MyUnique", "MyTable", type_="unique")
         context.assert_("ALTER TABLE `MyTable` DROP INDEX `MyUnique`")
 
     def test_drop_check_mariadb(self):
         context = op_fixture("mariadb")
-        op.drop_constraint("f1", "t1", "check")
+        op.drop_constraint("f1", "t1", type_="check")
         context.assert_("ALTER TABLE t1 DROP CONSTRAINT f1")
 
     def test_drop_check_quoted_mariadb(self):
         context = op_fixture("mariadb")
-        op.drop_constraint("MyCheck", "MyTable", "check")
+        op.drop_constraint("MyCheck", "MyTable", type_="check")
         context.assert_("ALTER TABLE `MyTable` DROP CONSTRAINT `MyCheck`")
 
     def test_drop_check_mysql(self):
         context = op_fixture("mysql")
-        op.drop_constraint("f1", "t1", "check")
+        op.drop_constraint("f1", "t1", type_="check")
         context.assert_("ALTER TABLE t1 DROP CHECK f1")
 
     def test_drop_check_quoted_mysql(self):
         context = op_fixture("mysql")
-        op.drop_constraint("MyCheck", "MyTable", "check")
+        op.drop_constraint("MyCheck", "MyTable", type_="check")
         context.assert_("ALTER TABLE `MyTable` DROP CHECK `MyCheck`")
 
     def test_drop_unknown(self):
@@ -425,7 +454,7 @@ class MySQLOpTest(TestBase):
             op.drop_constraint,
             "f1",
             "t1",
-            "typo",
+            type_="typo",
         )
 
     def test_drop_generic_constraint(self):
@@ -439,9 +468,50 @@ class MySQLOpTest(TestBase):
             "t1",
         )
 
+    @combinations(
+        (lambda: Computed("foo * 5"), lambda: None),
+        (lambda: None, lambda: Computed("foo * 5")),
+        (
+            lambda: Computed("foo * 42"),
+            lambda: Computed("foo * 5"),
+        ),
+    )
+    def test_alter_column_computed_not_supported(self, sd, esd):
+        op_fixture("mysql")
+        assert_raises_message(
+            exc.CompileError,
+            'Adding or removing a "computed" construct, e.g. '
+            "GENERATED ALWAYS AS, to or from an existing column is not "
+            "supported.",
+            op.alter_column,
+            "t1",
+            "c1",
+            server_default=sd(),
+            existing_server_default=esd(),
+        )
+
+    @combinations(
+        (lambda: Identity(), lambda: None),
+        (lambda: None, lambda: Identity()),
+        (lambda: Identity(), lambda: Identity()),
+    )
+    def test_alter_column_identity_not_supported(self, sd, esd):
+        op_fixture()
+        assert_raises_message(
+            exc.CompileError,
+            'Adding, removing or modifying an "identity" construct, '
+            "e.g. GENERATED AS IDENTITY, to or from an existing "
+            "column is not supported in this dialect.",
+            op.alter_column,
+            "t1",
+            "c1",
+            server_default=sd(),
+            existing_server_default=esd(),
+        )
+
 
 class MySQLBackendOpTest(AlterColRoundTripFixture, TestBase):
-    __only_on__ = "mysql"
+    __only_on__ = "mysql", "mariadb"
     __backend__ = True
 
     def test_add_timestamp_server_default_current_timestamp(self):
@@ -470,7 +540,7 @@ class MySQLBackendOpTest(AlterColRoundTripFixture, TestBase):
         )
 
     def test_add_timestamp_server_default_current_timestamp_bundle_onupdate(
-        self
+        self,
     ):
         # note SQLAlchemy reflection bundles the ON UPDATE part into the
         # server default reflection see
@@ -485,7 +555,7 @@ class MySQLBackendOpTest(AlterColRoundTripFixture, TestBase):
         )
 
     def test_add_datetime_server_default_current_timestamp_bundle_onupdate(
-        self
+        self,
     ):
         # note SQLAlchemy reflection bundles the ON UPDATE part into the
         # server default reflection see
@@ -501,11 +571,9 @@ class MySQLBackendOpTest(AlterColRoundTripFixture, TestBase):
 
 
 class MySQLDefaultCompareTest(TestBase):
-    __only_on__ = "mysql"
+    __only_on__ = "mysql", "mariadb"
     __backend__ = True
 
-    __requires__ = ("mysql_timestamp_reflection",)
-
     @classmethod
     def setup_class(cls):
         cls.bind = config.db
@@ -527,10 +595,11 @@ class MySQLDefaultCompareTest(TestBase):
         clear_staging_env()
 
     def setUp(self):
-        self.metadata = MetaData(self.bind)
+        self.metadata = MetaData()
 
     def tearDown(self):
-        self.metadata.drop_all()
+        with config.db.begin() as conn:
+            self.metadata.drop_all(conn)
 
     def _compare_default_roundtrip(self, type_, txt, alternate=None):
         if alternate:
@@ -559,7 +628,7 @@ class MySQLDefaultCompareTest(TestBase):
         insp = inspect(self.bind)
         cols = insp.get_columns(t1.name)
         refl = Table(t1.name, MetaData())
-        insp.reflecttable(refl, None)
+        insp.reflect_table(refl, include_columns=None)
         ctx = self.autogen_context["context"]
         return ctx.impl.compare_server_default(
             refl.c[cols[0]["name"]], col, rendered, cols[0]["default"]
