@@ -1,10 +1,14 @@
+import itertools
+
 from sqlalchemy import BigInteger
 from sqlalchemy import Boolean
 from sqlalchemy import Column
+from sqlalchemy import Computed
 from sqlalchemy import DateTime
 from sqlalchemy import exc
 from sqlalchemy import Float
 from sqlalchemy import func
+from sqlalchemy import Identity
 from sqlalchemy import Index
 from sqlalchemy import inspect
 from sqlalchemy import Integer
@@ -33,6 +37,7 @@ from sqlalchemy.sql.expression import literal_column
 from alembic import autogenerate
 from alembic import command
 from alembic import op
+from alembic import testing
 from alembic import util
 from alembic.autogenerate import api
 from alembic.autogenerate.compare import _compare_server_default
@@ -47,6 +52,9 @@ from alembic.testing import config
 from alembic.testing import eq_
 from alembic.testing import eq_ignore_whitespace
 from alembic.testing import provide_metadata
+from alembic.testing import resolve_lambda
+from alembic.testing import schemacompare
+from alembic.testing.assertions import expect_warnings
 from alembic.testing.env import _no_sql_testing_config
 from alembic.testing.env import clear_staging_env
 from alembic.testing.env import staging_env
@@ -57,7 +65,6 @@ from alembic.testing.fixtures import op_fixture
 from alembic.testing.fixtures import TablesTest
 from alembic.testing.fixtures import TestBase
 from alembic.testing.suite._autogen_fixtures import AutogenFixtureTest
-from alembic.util import sqla_compat
 
 
 class PostgresqlOpTest(TestBase):
@@ -109,7 +116,6 @@ class PostgresqlOpTest(TestBase):
             "CREATE INDEX CONCURRENTLY geocoded ON locations (coordinates)"
         )
 
-    @config.requirements.sqlalchemy_14
     def test_create_index_postgresql_include(self):
         context = op_fixture("postgresql")
         op.create_index(
@@ -122,7 +128,6 @@ class PostgresqlOpTest(TestBase):
         op.create_index("i", "t", ["c1", "c2"], unique=False)
         context.assert_("CREATE INDEX i ON t (c1, c2)")
 
-    @config.requirements.sqlalchemy_2
     def test_create_index_postgresql_if_not_exists(self):
         context = op_fixture("postgresql")
         op.create_index("i", "t", ["c1", "c2"], if_not_exists=True)
@@ -141,7 +146,6 @@ class PostgresqlOpTest(TestBase):
             op.drop_index("geocoded", postgresql_concurrently=True)
         context.assert_("DROP INDEX CONCURRENTLY geocoded")
 
-    @config.requirements.sqlalchemy_2
     def test_drop_index_postgresql_if_exists(self):
         context = op_fixture("postgresql")
         op.drop_index("geocoded", if_exists=True)
@@ -328,7 +332,7 @@ class PostgresqlOpTest(TestBase):
         context = op_fixture("postgresql")
         op.add_column(
             "t1",
-            Column("some_column", Integer, sqla_compat.Computed("foo * 5")),
+            Column("some_column", Integer, Computed("foo * 5")),
         )
         context.assert_(
             "ALTER TABLE t1 ADD COLUMN some_column "
@@ -336,11 +340,11 @@ class PostgresqlOpTest(TestBase):
         )
 
     @combinations(
-        (lambda: sqla_compat.Computed("foo * 5"), lambda: None),
-        (lambda: None, lambda: sqla_compat.Computed("foo * 5")),
+        (lambda: Computed("foo * 5"), lambda: None),
+        (lambda: None, lambda: Computed("foo * 5")),
         (
-            lambda: sqla_compat.Computed("foo * 42"),
-            lambda: sqla_compat.Computed("foo * 5"),
+            lambda: Computed("foo * 42"),
+            lambda: Computed("foo * 5"),
         ),
     )
     @config.requirements.computed_columns
@@ -371,7 +375,7 @@ class PostgresqlOpTest(TestBase):
         context = op_fixture("postgresql")
         op.add_column(
             "t1",
-            Column("some_column", Integer, sqla_compat.Identity(**kw)),
+            Column("some_column", Integer, Identity(**kw)),
         )
         qualification = "ALWAYS" if kw.get("always", False) else "BY DEFAULT"
         options = " (%s)" % text if text else ""
@@ -394,7 +398,7 @@ class PostgresqlOpTest(TestBase):
         op.alter_column(
             "t1",
             "some_column",
-            server_default=sqla_compat.Identity(**kw),
+            server_default=Identity(**kw),
             existing_server_default=None,
         )
         qualification = "ALWAYS" if kw.get("always", False) else "BY DEFAULT"
@@ -411,7 +415,7 @@ class PostgresqlOpTest(TestBase):
             "t1",
             "some_column",
             server_default=None,
-            existing_server_default=sqla_compat.Identity(),
+            existing_server_default=Identity(),
         )
         context.assert_(
             "ALTER TABLE t1 ALTER COLUMN some_column DROP IDENTITY"
@@ -458,8 +462,8 @@ class PostgresqlOpTest(TestBase):
         op.alter_column(
             "t1",
             "some_column",
-            server_default=sqla_compat.Identity(**updated),
-            existing_server_default=sqla_compat.Identity(**existing),
+            server_default=Identity(**updated),
+            existing_server_default=Identity(**existing),
         )
         context.assert_("ALTER TABLE t1 ALTER COLUMN some_column %s" % text)
 
@@ -853,8 +857,8 @@ class PostgresqlDetectSerialTest(TestBase):
         clear_staging_env()
 
     @provide_metadata
-    def _expect_default(self, c_expected, col, seq=None):
-        Table("t", self.metadata, col)
+    def _expect_default(self, c_expected, col, schema=None, seq=None):
+        Table("t", self.metadata, col, schema=schema)
 
         self.autogen_context.metadata = self.metadata
 
@@ -865,7 +869,7 @@ class PostgresqlDetectSerialTest(TestBase):
         insp = inspect(config.db)
 
         uo = ops.UpgradeOps(ops=[])
-        _compare_tables({(None, "t")}, set(), insp, uo, self.autogen_context)
+        _compare_tables({(schema, "t")}, set(), insp, uo, self.autogen_context)
         diffs = uo.as_diffs()
         tab = diffs[0][1]
 
@@ -878,12 +882,12 @@ class PostgresqlDetectSerialTest(TestBase):
 
         insp = inspect(config.db)
         uo = ops.UpgradeOps(ops=[])
-        m2 = MetaData()
+        m2 = MetaData(schema=schema)
         Table("t", m2, Column("x", BigInteger()))
         self.autogen_context.metadata = m2
         _compare_tables(
-            {(None, "t")},
-            {(None, "t")},
+            {(schema, "t")},
+            {(schema, "t")},
             insp,
             uo,
             self.autogen_context,
@@ -897,35 +901,47 @@ class PostgresqlDetectSerialTest(TestBase):
             c_expected,
         )
 
-    def test_serial(self):
-        self._expect_default(None, Column("x", Integer, primary_key=True))
-
-    def test_separate_seq(self):
-        seq = Sequence("x_id_seq")
+    @testing.combinations((None,), ("test_schema",))
+    def test_serial(self, schema):
         self._expect_default(
-            "nextval('x_id_seq'::regclass)",
+            None, Column("x", Integer, primary_key=True), schema
+        )
+
+    @testing.combinations((None,), ("test_schema",))
+    def test_separate_seq(self, schema):
+        seq = Sequence("x_id_seq", schema=schema)
+        seq_name = seq.name if schema is None else f"{schema}.{seq.name}"
+        self._expect_default(
+            f"nextval('{seq_name}'::regclass)",
             Column(
                 "x", Integer, server_default=seq.next_value(), primary_key=True
             ),
+            schema,
             seq,
         )
 
-    def test_numeric(self):
-        seq = Sequence("x_id_seq")
+    @testing.combinations((None,), ("test_schema",))
+    def test_numeric(self, schema):
+        seq = Sequence("x_id_seq", schema=schema)
+        seq_name = seq.name if schema is None else f"{schema}.{seq.name}"
         self._expect_default(
-            "nextval('x_id_seq'::regclass)",
+            f"nextval('{seq_name}'::regclass)",
             Column(
                 "x",
                 Numeric(8, 2),
                 server_default=seq.next_value(),
                 primary_key=True,
             ),
+            schema,
             seq,
         )
 
-    def test_no_default(self):
+    @testing.combinations((None,), ("test_schema",))
+    def test_no_default(self, schema):
         self._expect_default(
-            None, Column("x", Integer, autoincrement=False, primary_key=True)
+            None,
+            Column("x", Integer, autoincrement=False, primary_key=True),
+            schema,
         )
 
 
@@ -1312,7 +1328,7 @@ class PostgresqlAutogenRenderTest(TestBase):
                 ops.CreateIndexOp.from_index(idx),
             ),
             "op.create_index('my_idx', 'tbl', "
-            "[sa.text(\"(c ->> 'foo')\")], unique=False)",
+            "[sa.literal_column(\"(c ->> 'foo')\")], unique=False)",
         )
 
     @config.requirements.nulls_not_distinct_sa
@@ -1492,6 +1508,217 @@ class PGUniqueIndexAutogenerateTest(AutogenFixtureTest, TestBase):
         eq_(len(diffs), 1)
 
 
+def _lots_of_indexes(flatten: bool = False):
+    diff_pairs = [
+        (
+            lambda t: Index("idx", t.c.jb["foo"]),
+            lambda t: Index("idx", t.c.jb["bar"]),
+        ),
+        (
+            lambda t: Index("idx", t.c.jb["foo"]),
+            lambda t: Index("idx", t.c.jb["not_jsonb_path_ops"]),
+        ),
+        (
+            lambda t: Index("idx", t.c.jb["not_jsonb_path_ops"]),
+            lambda t: Index("idx", t.c.jb["bar"]),
+        ),
+        (
+            lambda t: Index("idx", t.c.aa),
+            lambda t: Index("idx", t.c.not_jsonb_path_ops),
+        ),
+        (
+            lambda t: Index("idx", t.c.not_jsonb_path_ops),
+            lambda t: Index("idx", t.c.aa),
+        ),
+        (
+            lambda t: Index(
+                "idx",
+                t.c.jb["foo"].label("x"),
+                postgresql_using="gin",
+                postgresql_ops={"x": "jsonb_path_ops"},
+            ),
+            lambda t: Index(
+                "idx",
+                t.c.jb["bar"].label("x"),
+                postgresql_using="gin",
+                postgresql_ops={"x": "jsonb_path_ops"},
+            ),
+        ),
+        (
+            lambda t: Index("idx", t.c.jb["foo"].astext),
+            lambda t: Index("idx", t.c.jb["bar"].astext),
+        ),
+        (
+            lambda t: Index("idx", t.c.jb["foo"].as_integer()),
+            lambda t: Index("idx", t.c.jb["bar"].as_integer()),
+        ),
+        (
+            lambda t: Index("idx", text("(jb->'x')"), _table=t),
+            lambda t: Index("idx", text("(jb->'y')"), _table=t),
+        ),
+    ]
+    if flatten:
+        return list(itertools.chain.from_iterable(diff_pairs))
+    else:
+        return diff_pairs
+
+
+def _equal_indexes():
+    the_indexes = [(fn, fn) for fn in _lots_of_indexes(True)]
+    the_indexes += [
+        (
+            lambda t: Index("idx", text("(jb->'x')"), _table=t),
+            lambda t: Index("idx", text("(jb -> 'x')"), _table=t),
+        ),
+        (
+            lambda t: Index("idx", text("cast(jb->'x' as integer)"), _table=t),
+            lambda t: Index("idx", text("(jb -> 'x')::integer"), _table=t),
+        ),
+    ]
+    return the_indexes
+
+
+def _index_op_clause():
+    def make_idx(t, *expr):
+        return Index(
+            "idx",
+            *(text(e) if isinstance(e, str) else e for e in expr),
+            postgresql_using="gin",
+            _table=t,
+        )
+
+    return [
+        (
+            False,
+            lambda t: make_idx(t, "(jb->'x')jsonb_path_ops"),
+            lambda t: make_idx(t, "(jb->'x')jsonb_path_ops"),
+        ),
+        (
+            False,
+            lambda t: make_idx(t, "aa array_ops"),
+            lambda t: make_idx(t, "aa array_ops"),
+        ),
+        (
+            False,
+            lambda t: make_idx(t, "(jb->'x')jsonb_path_ops"),
+            lambda t: make_idx(t, "(jb->'y')jsonb_path_ops"),
+        ),
+        (
+            False,
+            lambda t: make_idx(t, "aa array_ops"),
+            lambda t: make_idx(t, "jb array_ops"),
+        ),
+        (
+            False,
+            lambda t: make_idx(t, "aa array_ops", "(jb->'y')jsonb_path_ops"),
+            lambda t: make_idx(t, "(jb->'y')jsonb_path_ops", "aa array_ops"),
+        ),
+        (
+            True,
+            lambda t: make_idx(t, "aa array_ops", text("(jb->'x')")),
+            lambda t: make_idx(t, "aa array_ops", text("(jb->'y')")),
+        ),
+        (
+            True,
+            lambda t: make_idx(t, text("(jb->'x')"), "aa array_ops"),
+            lambda t: make_idx(t, text("(jb->'y')"), "aa array_ops"),
+        ),
+        (
+            True,
+            lambda t: make_idx(t, "aa array_ops", text("(jb->'x')")),
+            lambda t: make_idx(t, "jb array_ops", text("(jb->'y')")),
+        ),
+        (
+            True,
+            lambda t: make_idx(t, text("(jb->'x')"), "aa array_ops"),
+            lambda t: make_idx(t, text("(jb->'y')"), "jb array_ops"),
+        ),
+    ]
+
+
+class PGIndexAutogenerateTest(AutogenFixtureTest, TestBase):
+    __backend__ = True
+    __only_on__ = "postgresql"
+    __requires__ = ("reflect_indexes_with_expressions",)
+
+    @testing.fixture
+    def index_tables(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        t_old = Table(
+            "exp_index",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("aa", ARRAY(Integer)),
+            Column("jb", JSONB),
+            Column("not_jsonb_path_ops", Integer),
+        )
+
+        t_new = Table(
+            "exp_index",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("aa", ARRAY(Integer)),
+            Column("jb", JSONB),
+            Column("not_jsonb_path_ops", Integer),
+        )
+
+        return m1, m2, t_old, t_new
+
+    @combinations(*_lots_of_indexes(), argnames="old_fn, new_fn")
+    def test_expression_indexes_changed(self, index_tables, old_fn, new_fn):
+        m1, m2, old_table, new_table = index_tables
+
+        old = resolve_lambda(old_fn, t=old_table)
+        new = resolve_lambda(new_fn, t=new_table)
+
+        diffs = self._fixture(m1, m2)
+        eq_(
+            diffs,
+            [
+                ("remove_index", schemacompare.CompareIndex(old, True)),
+                ("add_index", schemacompare.CompareIndex(new)),
+            ],
+        )
+
+    @combinations(*_equal_indexes(), argnames="fn1, fn2")
+    def test_expression_indexes_no_change(self, index_tables, fn1, fn2):
+        m1, m2, old_table, new_table = index_tables
+
+        resolve_lambda(fn1, t=old_table)
+        resolve_lambda(fn2, t=new_table)
+
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
+    @combinations(*_index_op_clause(), argnames="changed, old_fn, new_fn")
+    def test_expression_indexes_warn_operator(
+        self, index_tables, changed, old_fn, new_fn
+    ):
+        m1, m2, old_table, new_table = index_tables
+
+        old = old_fn(t=old_table)
+        new = new_fn(t=new_table)
+
+        with expect_warnings(
+            r"Expression #\d .+ in index 'idx' detected to include "
+            "an operator clause. Expression compare cannot proceed. "
+            "Please move the operator clause to the "
+        ):
+            diffs = self._fixture(m1, m2)
+        if changed:
+            eq_(
+                diffs,
+                [
+                    ("remove_index", schemacompare.CompareIndex(old, True)),
+                    ("add_index", schemacompare.CompareIndex(new)),
+                ],
+            )
+        else:
+            eq_(diffs, [])
+
+
 case = combinations(
     ("nulls_not_distinct=False", False),
     ("nulls_not_distinct=True", True),
