@@ -4,7 +4,6 @@ from pathlib import Path
 import re
 from unittest.mock import patch
 
-from dateutil import tz
 import sqlalchemy as sa
 from sqlalchemy import Column
 from sqlalchemy import inspect
@@ -41,6 +40,11 @@ from alembic.testing.env import write_script
 from alembic.testing.fixtures import TestBase
 from alembic.util import CommandError
 
+try:
+    from zoneinfo import ZoneInfo
+except ImportError:
+    from backports.zoneinfo import ZoneInfo
+
 env, abc, def_ = None, None, None
 
 
@@ -189,12 +193,16 @@ class ScriptNamingTest(TestBase):
 
     @testing.combinations(
         (
-            datetime.datetime(2012, 7, 25, 15, 8, 5, tzinfo=tz.gettz("UTC")),
+            datetime.datetime(
+                2012, 7, 25, 15, 8, 5, tzinfo=datetime.timezone.utc
+            ),
             "%s/versions/1343228885_12345_this_is_a_"
             "message_2012_7_25_15_8_5.py",
         ),
         (
-            datetime.datetime(2012, 7, 25, 15, 8, 6, tzinfo=tz.gettz("UTC")),
+            datetime.datetime(
+                2012, 7, 25, 15, 8, 6, tzinfo=datetime.timezone.utc
+            ),
             "%s/versions/1343228886_12345_this_is_a_"
             "message_2012_7_25_15_8_6.py",
         ),
@@ -227,7 +235,8 @@ class ScriptNamingTest(TestBase):
         with mock.patch(
             "alembic.script.base.datetime",
             mock.Mock(
-                datetime=mock.Mock(utcnow=lambda: given, now=lambda: given)
+                datetime=mock.Mock(utcnow=lambda: given, now=lambda: given),
+                timezone=datetime.timezone,
             ),
         ):
             create_date = script._generate_create_date()
@@ -238,7 +247,7 @@ class ScriptNamingTest(TestBase):
             "EST5EDT",
             datetime.datetime(2012, 7, 25, 15, 8, 5),
             datetime.datetime(
-                2012, 7, 25, 11, 8, 5, tzinfo=tz.gettz("EST5EDT")
+                2012, 7, 25, 11, 8, 5, tzinfo=ZoneInfo("EST5EDT")
             ),
         )
 
@@ -247,7 +256,7 @@ class ScriptNamingTest(TestBase):
             "est5edt",
             datetime.datetime(2012, 7, 25, 15, 8, 5),
             datetime.datetime(
-                2012, 7, 25, 11, 8, 5, tzinfo=tz.gettz("EST5EDT")
+                2012, 7, 25, 11, 8, 5, tzinfo=ZoneInfo("EST5EDT")
             ),
         )
 
@@ -255,7 +264,7 @@ class ScriptNamingTest(TestBase):
         self._test_tz(
             "utc",
             datetime.datetime(2012, 7, 25, 15, 8, 5),
-            datetime.datetime(2012, 7, 25, 15, 8, 5, tzinfo=tz.gettz("UTC")),
+            datetime.datetime(2012, 7, 25, 15, 8, 5, tzinfo=ZoneInfo("UTC")),
         )
 
     def test_custom_tzdata_tz(self):
@@ -263,7 +272,7 @@ class ScriptNamingTest(TestBase):
             "Europe/Berlin",
             datetime.datetime(2012, 7, 25, 15, 8, 5),
             datetime.datetime(
-                2012, 7, 25, 17, 8, 5, tzinfo=tz.gettz("Europe/Berlin")
+                2012, 7, 25, 17, 8, 5, tzinfo=ZoneInfo("Europe/Berlin")
             ),
         )
 
@@ -284,10 +293,12 @@ class ScriptNamingTest(TestBase):
             datetime.datetime(2012, 7, 25, 15, 8, 5),
         )
 
-    def test_no_dateutil_module(self):
-        with patch("alembic.script.base.tz", new=None):
+    def test_no_zoneinfo_module(self):
+        with patch("alembic.script.base.ZoneInfo", new=None):
             with expect_raises_message(
-                CommandError, "The library 'python-dateutil' is required"
+                CommandError,
+                "Python >= 3.9 is required for timezone support or "
+                "the 'backports.zoneinfo' package must be installed.",
             ):
                 self._test_tz(
                     "utc",
@@ -646,8 +657,9 @@ def downgrade():
             result = handle.read()
         assert (
             (
-                """
+                '''
 def upgrade() -> None:
+    """Upgrade schema."""
     # ### commands auto generated by Alembic - please adjust! ###
     op.create_table('test_table',
     sa.Column('id', sa.Integer(), nullable=False),
@@ -655,7 +667,7 @@ def upgrade() -> None:
     sa.PrimaryKeyConstraint('id')
     )
     # ### end Alembic commands ###
-"""
+'''
             )
             in result
         )
@@ -922,6 +934,11 @@ class RewriterTest(TestBase):
             idx_op = ops.CreateIndexOp("ixt", op.table_name, [op.column.name])
             return [op, idx_op]
 
+        def process_revision_directives(context, revision, generate_revisions):
+            generate_revisions[0].downgrade_ops = ops.DowngradeOps(
+                ops=[ops.DropColumnOp("t1", "x")]
+            )
+
         directives = [
             ops.MigrationScript(
                 util.rev_id(),
@@ -945,7 +962,8 @@ class RewriterTest(TestBase):
         ]
 
         ctx, rev = mock.Mock(), mock.Mock()
-        writer1.chain(writer2)(ctx, rev, directives)
+        writer = writer1.chain(process_revision_directives).chain(writer2)
+        writer(ctx, rev, directives)
 
         eq_(
             autogenerate.render_python_code(directives[0].upgrade_ops),
@@ -959,6 +977,13 @@ class RewriterTest(TestBase):
             "    # ### end Alembic commands ###",
         )
 
+        eq_(
+            autogenerate.render_python_code(directives[0].downgrade_ops),
+            "# ### commands auto generated by Alembic - please adjust! ###\n"
+            "    op.drop_column('t1', 'x')\n"
+            "    # ### end Alembic commands ###",
+        )
+
     def test_no_needless_pass(self):
         writer1 = autogenerate.Rewriter()
 
@@ -1353,7 +1378,7 @@ class NormPathTest(TestBase):
         script = ScriptDirectory.from_config(config)
 
         def normpath(path):
-            return path.replace("/", ":NORM:")
+            return path.replace(os.pathsep, ":NORM:")
 
         normpath = mock.Mock(side_effect=normpath)
 
@@ -1365,7 +1390,7 @@ class NormPathTest(TestBase):
                         os.path.join(
                             _get_staging_directory(), "scripts", "versions"
                         )
-                    ).replace("/", ":NORM:"),
+                    ).replace(os.pathsep, ":NORM:"),
                 ),
             )
 
@@ -1375,7 +1400,7 @@ class NormPathTest(TestBase):
                     os.path.join(
                         _get_staging_directory(), "scripts", "versions"
                     )
-                ).replace("/", ":NORM:"),
+                ).replace(os.pathsep, ":NORM:"),
             )
 
     def test_script_location_multiple(self):
@@ -1384,7 +1409,7 @@ class NormPathTest(TestBase):
         script = ScriptDirectory.from_config(config)
 
         def _normpath(path):
-            return path.replace("/", ":NORM:")
+            return path.replace(os.pathsep, ":NORM:")
 
         normpath = mock.Mock(side_effect=_normpath)
 
